@@ -739,6 +739,10 @@ gst_tag_list_new_empty (void)
  * function. The tag list will make copies of any arguments passed
  * (e.g. strings, buffers).
  *
+ * After creation you might also want to set a #GstTagScope on the returned
+ * taglist to signal if the contained tags are global or stream tags. By
+ * default stream scope is assumes. See gst_tag_list_set_scope().
+ *
  * Free-function: gst_tag_list_unref
  *
  * Returns: (transfer full): a new #GstTagList. Free with gst_tag_list_unref()
@@ -792,7 +796,7 @@ gst_tag_list_new_valist (va_list var_args)
  * @scope: new scope for @list
  *
  * Sets the scope of @list to @scope. By default the scope
- * of a taglist is #GST_TAG_SCOPE_STREAM.
+ * of a taglist is stream scope.
  *
  */
 void
@@ -1257,7 +1261,13 @@ gst_tag_list_add_valist (GstTagList * list, GstTagMergeMode mode,
        */
       return;
     }
-    gst_tag_list_add_value_internal (list, mode, tag, &value, info);
+    /* Facilitate GstBuffer -> GstSample transition */
+    if (G_UNLIKELY (info->type == GST_TYPE_SAMPLE &&
+            !GST_IS_SAMPLE (value.data[0].v_pointer))) {
+      g_warning ("Expected GstSample argument for tag '%s'", tag);
+    } else {
+      gst_tag_list_add_value_internal (list, mode, tag, &value, info);
+    }
     g_value_unset (&value);
     tag = va_arg (var_args, gchar *);
   }
